// Магазинными символами автомата являются S, B, маркер дна (botoom).
// Их значение должно быть за пределами видимых символов.
var S = 25600;
var B = 25700;
var bottom = 25800;

var stck = []; // Стек для магазинных символов

var inStr = ""; // Строка имитирующая входную ленту (данные из формы)
var erStr = ""; // Строка с сообщением о результатах разбора
var i = 0;  // Текущее положение входной головки
var erFlag = -1; // Флаг, фиксирующий наличие ошибок в середине правила

// Функция, реализующая чтение символов в буфер из поля формы.
function GetLine() {
    var bracketsInput = document.getElementById("bracketsInput");
    inStr = bracketsInput.value + "\n";
}

// Инициализация устройств АМП
function Init() {
    // Инициализация стека
    stck = [];
    stck.push(bottom);
    stck.push(S);

    // Инициализация входной головки
    i = 0;
}

// Элемент на вершине стека
function getTop() {
    return stck[stck.length-1];
}

// Замена элемента на вершине стека
// эквивалентно s.pop(); s.push(B);
function setTop(e) {
    stck.pop();
    stck.push(e)
    //s[s.length-1] = e;
}

// Вывод значения магазинного символа вместо числа
function stackSymbol(n) {
    var ch;
    if(n === 25600) ch = "S";
    else if(n === 25700) ch = "B";
    else if(n === 25800) ch = "bottom";
    else ch = n;
    return ch;
}

// Вывод конечного символа входной цепочки
function lastSymbol(c) {
    var ch;
    if(c === "\n") ch = "\\n";
    else ch = c;
    return ch;
}

// Устройство управления АМП, анализирующего вложенность скобок.
// Имитирует таблицу переходов АМП.
function Parser() {
    // Инициализация устройств АМП
    Init();

    var step = 0;
    // Цикл анализа состояний
    while(true) {
        var top = getTop();
        // Тестовый вывод информации о текущем шаге, текущем символе, вершине стека
        erStr +="<br>step " + (step++) + ": inStr[" + i + "] = "+ lastSymbol(inStr[i]) + " Top = "+ stackSymbol(top);
        // Проверка содержимого на вершине стека
        switch(top) {
            // Анализ первой строки таблицы переходов
            case S:
                switch(inStr[i]) {
                    case "(": // [S, (]
                        setTop(B); // эквивалентно stck.pop(); stck.push(B);
                        stck.push(")");
                        stck.push(B);
                        i++; // следующий входной символ
                        break;
                    case ")": // [S, )] - ) не может стоять в начале
                        erStr += "<br>Position " + i + ", " + "Error 1: \")\" can not is in begin!";
                        return false;
                    case "\n": // [S, концевой маркер]
                        erStr += "<br>Position " + i + ", " + "Error 2: Empty string!";
                        return false;
                    default: // Ошибка, такой символ не допустим
                        erStr += "<br>Position " + i + ", " + "Incorrect symbol! " + inStr[i];
                        return false;
                }
                break;

            // Анализ второй строки таблицы переходов
            case B:
                switch(inStr[i]) {
                    case "(": // [B, (]
                        setTop(B); // эквивалентно stck.pop(); stck.push(B);
                        stck.push(")");
                        stck.push(B);
                        i++; // следующий входной символ
                        break;
                    case ")": // [B, )] - вытолкнуть без сдвига 
                        stck.pop();
                        break;
                    case "\n": // [B, концевой маркер] - вытолкнуть без сдвига
                        stck.pop();
                        break;
                    default: // Ошибка, такой символ не допустим
                        erStr += "<br>Position " + i + ", " + "Incorrect symbol! " + inStr[i];
                        return false;
                }
                break;
            
            // Анализ третьей строки таблицы переходов
            case ")":
                switch(inStr[i]) {
                    case "(": // [), (] - в принципе недостижимо
                        erStr += "<br>Position " + i + ", " + "Error 3: I want \")\"!";
                        return false;
                    case ")": // [), )] - вытолкнуть со сдвигом 
                        stck.pop(); // вытолкнуть
                        i++;      // сдвиг
                        break;
                    case "\n": // [), концевой маркер]
                        erStr += "<br>Position " + i + ", " + "Error 4: I want \")\"!";
                        return false;
                    default: // Ошибка, такой символ не допустим
                        erStr += "<br>Position " + i + ", " + "Incorrect symbol! " + inStr[i];
                        return false;
                }
                break;

            // Анализ четвертой строки таблицы переходов
            case bottom:
                switch(inStr[i]) {
                    case "(": // [bottom, (] - невозможно при пустом магазине
                        erStr += "<br>Position " + i + ", " + "Error 4: Impossible situation [bottom, \"(\"]!";
                        return false;
                    case ")": // [bottom, )] - лишняя )
                        erStr += "<br>Position " + i + ", " + "Error 5: unnecessary \")\"!";
                        return false;
                    case "\n": // [bottom, концевой маркер] - допустить!
                        return true;
                    default: // Ошибка, такой символ не допустим
                        erStr += "<br>Position " + i + ", " + "Incorrect symbol! " + inStr[i] + "";
                        return false;
                }
                break;
        }
    }
}

// Вспомогательная функция, осуществляющая повторение строки ch k раз.
function Repeat(k, ch) {
    var s = "";
    for(i = 0; i < k; i++) {
        s += ch;
    }
    return s;
}

// event handlers

// Функция запускающая парсер и осуществляющая вывод результатов разбора
function handleParseButton() {

    // Очистка буфера сообщений об ошибках
    erStr = "";
    // Чтение строки из формы в буфер для разбора
    GetLine();
    
    var rezStr = "";
    // Здесь начинается разбор принятой строки.
    if(Parser()) {
        rezStr = "+++++ OK! +++++";
    }
    else {
        rezStr = "----- Fatal error (look upper error message)! -----";
    }

    // Вывод разобранной строки и значения позиции входой головки.
    var mesStr = "Line: " + inStr;
    var strCursor = " Pos: " + Repeat(i, "-") + "^" + "  i = " + i;
    
    // Вывод сообщения о результатах разбора
    var messageField = document.getElementById("messageField");
    messageField.innerHTML = rezStr;

    // Вывод разобранного скобочного выражения
    var bracketsField = document.getElementById("bracketsField");
    bracketsField.innerHTML = mesStr;

    // Вывод позиции входной головки в момент окончания разбора
    var cursorField = document.getElementById("cursorField");
    cursorField.innerHTML = strCursor;

    // Вывод накопленных сообщений об ошибках к моменту окончания разбора
    var errorsField = document.getElementById("errorsField");
    errorsField.innerHTML = erStr;

}

function handleKeyPress(e) {
    var parseButton = document.getElementById("parseButton");

    // in IE9 and earlier, the event object doesn"t get passed
    // to the event handler correctly, so we use window.event instead.
    e = e || window.event;
    
    if (e.keyCode === 13) {
        parseButton.click();
        return false;
    }
}


// init - called when the page has completed loading
window.onload = init;

function init() {
    // Parse button onclick handler
    var parseButton = document.getElementById("parseButton");
    parseButton.onclick = handleParseButton;

    // handle "return" key press
    var bracketsInput = document.getElementById("bracketsInput");
    bracketsInput.onkeypress = handleKeyPress;
}
